unit HP54200A;
(* Delphi Example Program: NI-488.2 Direct Entry Points                *)
(* Example Program using NI-488.2 subroutines                          *)

(* ================================================================= *
 * This sample program is for reference only. It can only be expected
 * to function unmodified with an HP54200A Digital Oscilloscope, but changes
 * to accomodate other digital scopes using NI 488 interface boards should
 * be straightforward.  Note that the HP54200A is an old IEEE scope (1985),
 * and as such some commands, such as "IDN" don't work and have been changed.
 * Modified by TF 1/23/03
 * from code supplied by NI for Fluke 45 Digital Multimeter.
 *
 * The status variables IBSTA, IBERR, and IBCNTL are defined in the
 * VAR section below. Each bit of IBSTA and each value of IBERR are
 * defined in the CONST section below as a mnemonic constant for easy
 * recognition in application programs. In this example, these mnemonic
 * definitions are logically ANDed with the variable IBSTA to determine
 * if a particular bit has been set. The mnemonic definitions are
 * equated with the variable IBERR to determine the error code.
 *
 * The buttons displayed control the program's actions; they are self-
 * explanatory.
 *
 * The procedure GPIBERR is called when a NI-488.2 function fails. The
 * error message is printed along with the status variables IBSTA, IBERR,
 * and IBCNTL in message pop up dialog boxes. The procedure DVMERR is
 * called when the serial poll response byte indicates the Fluke 45 does
 * not have valid data to send. The error message and the serial poll
 * response byte are printed in a message pop up dialog box.
 *
 * The NI-488 function ibonl is called from the TForm1.Button1Click
 * procedure and from the two procedures, GPIBERR and DVMERR. When
 * the second parameter of the function ibonl is zero, the software
 * and the hardware are disabled. Execution of this program is terminated
 * after the call to the function ibonl to disable the software and
 * hardware.
 *
 * The function HALT is used to terminate this program within the
 * procedures GPIBERR and DVMERR.
 * ================================================================== *)

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  QStdCtrls, ExtCtrls, StdCtrls;

type
  TForm1 = class(TForm)        (* RUN button                               *)
    Button2: TButton;          (* QUIT button                              *)
    Button3: TButton;          (* ABOUT button                             *)
    ListBox1: TListBox;
    Label1: TLabel;
    PaintBox1: TPaintBox;
    RadioGroup1: TRadioGroup;
    RadioGroup2: TRadioGroup;
    Button1: TButton;
    ComboBox1: TComboBox;
    Label2: TLabel;
    Button4: TButton;
    RadioGroup3: TRadioGroup;
    Button5: TButton;
    RadioGroup4: TRadioGroup;
    RadioGroup5: TRadioGroup;
    RadioGroup6: TRadioGroup;
    RadioGroup7: TRadioGroup;
    RadioGroup8: TRadioGroup;
    RadioGroup9: TRadioGroup;
    Label4: TLabel;
    Label3: TLabel;
    ComboBox2: TComboBox;
    ComboBox3: TComboBox;
    Label5: TLabel;
    procedure initialize_hp54200A;
    Procedure Get_Information_About_Waveform ( Waveform_Number: integer );
    procedure Button1Click(Sender: TObject); (* Procedure to Refresh display *)
    procedure Button2Click(Sender: TObject); (* Procedure to Exit.           *)
    procedure Button3Click(Sender: TObject); (* Gives ABOUT info             *)
    procedure Button5Click(Sender: TObject); (* To allow scope autoscale *)
    procedure Display_graph {PaintBox1Paint}(Sender: TObject); (* Draws graph *)
    procedure RadioGroup1Click (Sender: Tobject); (* Handles channel 1 display *)
    procedure RadioGroup2Click (Sender: Tobject); (* Handles channel 2 display *)
    procedure RadioGroup3Click (Sender: Tobject); (* Handles mixed display *)
    procedure RadioGroup4Click (Sender: Tobject); (* Handles ch 1 norm/invert *)
    procedure RadioGroup5Click (Sender: Tobject); (* Handles ch 1 AC/DC coupling *)
    procedure RadioGroup6Click (Sender: Tobject); (* Handles ch 2 norm/invert *)
    procedure RadioGroup7Click (Sender: Tobject); (* Handles ch 2 AC/DC coupling *)
    procedure RadioGroup8Click (Sender: TObject ); (* Switches averaging on/off for ch 1 *)
    procedure RadioGroup9Click (Sender: TObject ); (* Switches averaging on/off for ch 2 *)
    procedure Get_output_format (Sender: TObject); (* finds out desired output format *)
    procedure Output_currently_active_graphs (Sender: TObject);  (* write out graph data *)
    procedure send_message (message_to_send: string);
    procedure Get_ch_1_averaging_times (Sender: TObject); (* Gets number of times to average ch 1 waveform *)
    procedure Get_ch_2_averaging_times (Sender: TObject); (* Gets number of times to average ch 2 waveform *)

 private
    { Private declarations }
  public
    { Public declarations }
  end;


const
  (* GPIB status bit definitions.                                        *)
  ERR     = $8000;           (* Error detected                           *)
  TIMO    = $4000;           (* Timeout                                  *)
  ENDgpib = $2000;           (* EOI or EOS detected                      *)
  SRQI    = $1000;           (* SRQ detected by CIC                      *)
  RQS     =  $800;           (* Device needs service                     *)
  SPOLL   =  $400;           (* Board has been serially polled           *)
  EVENT   =  $200;           (* An event has occurred                    *)
  CMPL    =  $100;           (* I/O completed                            *)
  LOK     =   $80;           (* Local lockout state                      *)
  REM     =   $40;           (* Remote state                             *)
  CIC     =   $20;           (* Controller-in-charge                     *)
  ATN     =   $10;           (* Attention asserted                       *)
  TACS    =    $8;           (* Talker active                            *)
  LACS    =    $4;           (* Listener active                          *)
  DTAS    =    $2;           (* Device trigger state                     *)
  DCAS    =    $1;           (* Device clear state                       *)

  (* Error messages returned in global variable IBERR:                   *)
  EDVR =  0;                 (* System error                             *)
  ECIC =  1;                 (* Function requires GPIB board to be CIC   *)
  ENOL =  2;                 (* Write function detected no Listeners     *)
  EADR =  3;                 (* Interface board not addressed correctly  *)
  EARG =  4;                 (* Invalid argument to function call        *)
  ESAC =  5;                 (* Function requires GPIB board to be SAC   *)
  EABO =  6;                 (* I/O operation aborted                    *)
  ENEB =  7;                 (* Non-existent interface board             *)
  EDMA =  8;                 (* Error performing DMA                     *)
  EOIP = 10;                 (* I/O operation started before previous    *)
			     (* operation completed                      *)
  ECAP = 11;                 (* No capability for intended operation     *)
  EFSO = 12;                 (* File system operation error              *)
  EBUS = 14;                 (* Command error during device call         *)
  ESTB = 15;                 (* Serial poll status byte lost             *)
  ESRQ = 16;                 (* SRQ remains asserted                     *)
  ETAB = 20;                 (* The return buffer is full                *)

  (* Values for the 488.2 Send command.                                  *)
  NULLend = $00;             (* Do nothing at the end of a transfer.     *)
  NLend   = $01;             (* Send NL with EOI after a transfer.       *)
  DABend  = $02;             (* Send EOI with the last DAB.              *)
  STOPend = $100;            (* Value used by the 488.2 Receive command. *)

  MAVbit  = $10;             (* Position of the Message Available bit.   *)

  (* Timeout values and meanings:                                        *)
  TNONE   =  0;              (* Infinite timeout (disabled)              *)
  T10us   =  1;              (* Timeout of 10 microseconds (ideal)       *)
  T30us   =  2;              (* Timeout of 30 microseconds (ideal)       *)
  T100us  =  3;              (* Timeout of 100 microseconds (ideal)      *)
  T300us  =  4;              (* Timeout of 300 microseconds (ideal)      *)
  T1ms    =  5;              (* Timeout of 1 milliseconds (ideal)        *)
  T3ms    =  6;              (* Timeout of 3 milliseconds (ideal)        *)
  T10ms   =  7;              (* Timeout of 10 milliseconds (ideal)       *)
  T30ms   =  8;              (* Timeout of 30 milliseconds (ideal)       *)
  T100ms  =  9;              (* Timeout of 100 milliseconds (ideal)      *)
  T300ms  = 10;              (* Timeout of 300 milliseconds (ideal)      *)
  T1s     = 11;              (* Timeout of 1 second (ideal)              *)
  T3s     = 12;              (* Timeout of 3 seconds (ideal)             *)
  T10s    = 13;              (* Timeout of 10 seconds (ideal)            *)
  T30s    = 14;              (* Timeout of 30 seconds (ideal)            *)
  T100s   = 15;              (* Timeout of 100 seconds (ideal)           *)
  T300s   = 16;              (* Timeout of 300 seconds (ideal)           *)
  T1000s  = 17;              (* Timeout of 1000 seconds (ideal)          *)

type
  AddrList = array [0..31] of SmallInt;

  (* Type declarations for exported NI-488.2 Global Variables.             *)
  Tibsta  = function : integer; stdcall;
  Tiberr  = function : integer; stdcall;
  Tibcntl = function : Longint; stdcall;

  (* Type declarations for exported NI-488.2 routines.                     *)
   TDevClear = procedure (boardID: integer;
  		                    address: word); stdcall;

   TFindLstn = procedure (boardID:    integer;
                          padlist:    AddrList;
                          resultlist: AddrList;
                          limit:      integer); stdcall;

   Tibclr = function (ud : integer) : integer; stdcall;

   Tibdev = function (ud:integer;
                      pad:integer;
                      sad:integer;
                      tmo:integer;
                      eot:integer;
                      eos:integer)  :  integer; stdcall;

   Tibonl = function(ud: integer;
                      v:  integer) : integer; stdcall;

   Tibrd  = function (ud:  integer;
                      var  rdbuf;
                      cnt: Longint) : integer; stdcall;

   Tibwrt = function (ud:  integer;
                      var  wrtbuf;
                      cnt: longint) : integer; stdcall;

   TReadStatusByte = procedure (boardID:    integer;
                                address:    word;
                               var result: integer); stdcall;

   TReceive = procedure (boardID:     integer;
      	                 address:     word;
	                       var buffer;
	                       cnt:         longint;
	                       termination: integer); stdcall;

   TSend = procedure (boardID: integer;
      	              address: word;
	                    var buffer;
	                    datacnt: longint;
	                    eotmode: integer); stdcall;

   TSendIFC = procedure (boardID: integer); stdcall;

   TWaitSRQ = procedure (boardID:    integer;
                         var result: integer); stdcall;
   data_byte_array = packed array[0..2048] of byte;
   data_word_array = packed array[0..2048] of word;
   data_buffer = packed array [0..180] of char;
   data_point_array = array[1..2,0..2048] of real;
   Waveform_graph = record
                      x_origin:           real;
                      y_origin:           real;
                      delta_x:            real;
                      delta_y:            real;
                      x_min:              real;
                      x_max:              real;
                      y_min:              real;      { based on scope-supplied y_reference_zero }
                      y_max:              real;
                      y_min_actual:       real;      { based on measurements of waveform }
                      y_max_actual:       real;
                      Y_average:          real;      { used in case of dc offset }
                      y_max_clipped:      boolean;   { true if actual values outside of computed range }
                      y_min_clipped:      boolean;
                      channel_inverted:   boolean;
                      coupling:           integer;
                      line_color:         integer;
                      number_of_x_lines:  integer;
                      number_of_y_lines:  integer;
                      y_reference_zero:   longint;
                      averaging:          boolean;
                      number_to_average:  integer;
                      valid_measurement_data: boolean;   { if true then following measurements ok }
                      rise_time:          real;
                      fall_time:          real;
                      period:             real;
                      frequency:          real;
                      rms_value:          real;
                      data_points:        data_point_array;
                    end;

   FFT_graph = record
                      x_origin:           real;
                      y_origin:           real;
                      delta_x:            real;
                      delta_y:            real;
                      x_min:              real;
                      x_max:              real;
                      y_min:              real;
                      y_max:              real;
                      line_color:         integer;
                      number_of_x_lines:  integer;
                      number_of_y_lines:  integer;
                      interpolation:      boolean;
                      data_points:        data_point_array;
                    end;

const
  BDINDEX               = 0;     (* Board Index                     *)
  PRIMARY_ADDR_OF_SCOPE = 5;     (* Primary address of HP54200A     *)
  NO_SECONDARY_ADDR     = 0;     (* Secondary address of HP54200A   *)
  TIMEOUT               = T3s;   (* Timeout value = 10 seconds      *)
  EOTMODE               = 1;     (* Enable the END message          *)
  EOSMODE               = 0;     (* Disable the EOS mode            *)

  ARRAYSIZE             = 1024;  (* Size of read buffer             *)
  Command_delay         = 100;   (* delay in ms for HP54200A commands *)
  max_number_to_average = 50;     (* number of waveforms to average *)
  adjustment_factor     = 1.05;  (* used to adjust graphing range so waveform doesn't go to screen edge *)

var
  Form1: TForm1;
  instruments : AddrList;
  results : AddrList;

(* NI-488.2 GPIB global status variables                                 *)
  AddrIbsta  : Tibsta;
  AddrIberr  : Tiberr;
  AddrIbcntl : Tibcntl;

(* Pointers to the NI-488.2 GPIB global status variables                 *)
  Pibsta : ^integer;
  Piberr : ^integer;
  Pibcntl : ^Longint;

(* Declaration for the Handle for the GPIB library                       *)
  Gpib32Lib: THandle;

(* Declarations for the NI-488.2 GPIB calls                              *)
  DevClear: TDevClear;
  FindLstn: TFindLstn;
  ReadStatusByte: TReadStatusByte;
  Receive: TReceive;
  Send: TSend;
  SendIFC: TSendIFC;
  WaitSRQ: TWaitSRQ;

(* Declarations for the NI-488.2 GPIB calls.                     *)
   ibclr : Tibclr;
   ibdev : Tibdev;
   ibrd  : Tibrd;
   ibwrt : Tibwrt;
   ibonl : Tibonl;

(* Declaration of global variables.                              *)


(* Declarations for HP54200A *)
  Dev                : integer;   (* device handle *)
  VStr               : packed array[0..ARRAYSIZE] of char;
  ValueStr           : packed array[0..ARRAYSIZE] of char;
  msg                : string;    (* String used for displaying messages.     *)
  format_parameter   : string;
  type_parameter     : string;
  graph_source       : string;     (* source of graph whose data being sent *)
  valid_data         : string;     (* info about whether data is valid graph *)
  points             : integer;    (* number of points in graph *)
  frame_count        : integer;    (* number of frames acquired or averaged *)
  xincrement         : real;       (* time delta between samples *)
  yincrement         : real;       (* volt delta value *)
  xorigin            : real;       (* initial time *)
  yorigin            : real;       (* initial y value *)
  xreference         : longint;    (* abscissa reference line display *)
  yreference         : longint;    (* ordinate reference line display *)
  coupling_parameter : string;     (* AC (2) or DC (1) coupling *)
  waveform_label     : string;     (* ID string for waveform *)
  byte_array         : array[1..max_number_to_average] of data_byte_array;   (* used if data from HP in byte format *)
  word_array         : array[1..max_number_to_average] of data_word_array;   (* used if data from HP in word format *)
  waveform_data      : array[1..2] of waveform_graph;   (* holds graph data; 1=ch 1
                                                           2=ch 2 *)
  fft_data           : array[1..2] of fft_graph;  (* holds fft data, 1=ch 1, 2=ch 2 *)
  data_in_byte_format: boolean; (* true if data in byte format, false if word format *)
  x_min, x_max       : real;     (* min, max values on abcissa to display *)
  y_min, y_max       : real;     (* min, max values on ordinate to display *)
  channel_1_display  : boolean = false;  (* true while channel 1 to be displayed *)
  channel_2_display  : boolean = false;  (* true while channel 2 to be displayed *)
  channel_1_invert   : boolean = false;  (* true while channel 1 to be inverted *)
  channel_2_invert   : boolean = false;  (* true while channel 2 to be inverted *)
  channel_sum        : boolean;          (* true while sum of channels to be displayed *)
  channel_1_FFT      : boolean = false;  (* true while channel 1 FFT to be displayed *)
  channel_2_FFT      : boolean = false;  (* true while channel 2 FFT to be displayed *)
  first_time         : boolean;          (* true if first time display is shown; reset when new data come *)
  channel_1_first_display : boolean = true;  (* false after any display of channel 1 *)
  channel_2_first_display : boolean = true;  (* false after any display of channel 2 *)
  old_data_points    : data_point_array; (* used for erasing old line *)
  initialized        : boolean = false;  (* make sure initialization routine has been done *)
  raw_data: array[0..2002] of longint;   (* raw data from scope - integer 0 to 32K *)
  metafile_output    : boolean = false;  (* true to output graph in WMF format *)
  copy_to_clipboard  : boolean = false;  (* true to copy graph to clipboard *)
  textfile_output    : boolean = false;  (* true to output raw data in standard text file *)
  output_format_selected: boolean = false; (* true if user has selected an output format *)
  mixed_display_mode : integer = 0;      (* set > 0 if both channels on one graph *)
  harmonics_to_display: integer;         (* arbitrary; later to be read from user *)

  waveform_subsystem_addressed: boolean = false; (* true if WAVEFORM command issued,
                                            allowing other Waveform subsystem commands *)
  channel_subsystem_addressed: boolean = false; (* true if channel subsystem active *)
  measure_subsystem_addressed: boolean = false; (* true if measure subsystem active  ref HP manual p. 8-73 *)

  multiplier: real;                            (* used to adjust time display *)
  paintbox_initialized: boolean = false;  (* set true after paintbox event handler done once *)

implementation

uses xygraph;  { contains the graphing routines }

{$R *.DFM}

(* =====================================================================
 *                      Functions min, max
 * These functions compute min and max of two real numbers
 *
 * ===================================================================== *)

function min (a,b: real): real;

begin
   if a >= b then min := b else min := a;
end;


function max (a,b: real): real;

begin
  if a >= b then max := a else max := b;
end;


(* ============================================================================
 *                Function Convert_number_to_string
 *  This function converts a number to a string, and labels it as sec, ms, microsec, or ns
 *  Can be preceded by a text item
 * =========================================================================== *)

function Convert_number_to_string ( text_item: string; number_to_convert: real ) : string;

var output_string: string;
    time_scale: string;
    temp: int64;

begin
   if number_to_convert > 1.1e6
      then begin
           time_scale := ' M';
           multiplier := 1e-6;
          end;
   if (number_to_convert <= 1.1e6 ) and (number_to_convert > 1.1e3)
      then begin
           time_scale := ' k';
           multiplier := 1e-3;
          end;
   if (number_to_convert <= 1100 ) and (number_to_convert > 1.1)
      then begin
           time_scale := ' ';
           multiplier := 1;
          end;
   if (number_to_convert <= 1.1) AND (number_to_convert > 1.1e-3)
      then begin
           time_scale := ' m';
           multiplier := 1000;
           end;
   if (number_to_convert <= 1.1e-3) and ( number_to_convert > 1.1e-6)
      then begin
           time_scale := ' ';
           multiplier := 1e6;
           end;
   if (number_to_convert < 1.1e-6)
       then begin
            time_scale := ' n';
            multiplier := 1e9;
            end;
   number_to_convert := number_to_convert * multiplier;
   temp := round ( number_to_convert*1000 );
   number_to_convert := temp;
   convert_number_to_string := text_item + FloatToStr(number_to_convert/1000.0) + time_scale;
end;


(* =====================================================================
 *                      Procedure loadDLL
 * This procedure loads the GPIB-32.DLL library during initialization.
 * If the LoadLibrary call is successful, the next step is to get the
 * addresses of the global status variables and functions using
 * GetProcAddress. If the GetProcAddress calls were successful, the
 * procedure returns to the main routine. Otherwise, it will free the
 * library and HALT.
 *
 * The HALT function will terminate this program.
 * ===================================================================== *)

procedure loadDLL;
var
  str : string;

  begin
  (* Load the GPIB-32.DLL library using the LoadLibrary function.        *)
    Gpib32Lib := LoadLibrary('GPIB-32.DLL');

  (* Check to see if library loaded successfully. If the library could   *)
  (* not be loaded, display an error message and then HALT the program.  *)
    If Gpib32Lib = 0 Then
      Begin
        str := 'LoadLibrary FAILED!';
        MessageDlg(str, mtError, [mbOK], 0);
        halt;
      End;
  (* Get the addresses of the GPIB Global Variables.                     *)
    @AddrIbsta  := GetProcAddress(Gpib32Lib, 'user_ibsta');
    @AddrIberr  := GetProcAddress(Gpib32Lib, 'user_iberr');
    @AddrIbcntl := GetProcAddress(Gpib32Lib, 'user_ibcnt');

  (* Get the addresses of the functions needed for this application.     *)
    @DevClear       := GetProcAddress(Gpib32Lib, 'DevClear');
    @FindLstn       := GetProcAddress(Gpib32Lib, 'FindLstn');
    @ReadStatusByte := GetProcAddress(Gpib32Lib, 'ReadStatusByte');
    @SendIFC        := GetProcAddress(Gpib32Lib, 'SendIFC');
    @WaitSRQ        := GetProcAddress(Gpib32Lib, 'WaitSRQ');
    @ibclr          := GetProcAddress(Gpib32Lib, 'ibclr');
    @ibdev          := GetProcAddress(Gpib32Lib, 'ibdev');
    @ibonl          := GetProcAddress(Gpib32Lib, 'ibonl');
    @ibrd           := GetProcAddress(Gpib32Lib, 'ibrd');
    @ibwrt          := GetProcAddress(Gpib32Lib, 'ibwrt');

  (* Verify that addresses were obtained. If unable to get any one of    *)
  (* the addresses, then free the library, display an error message and  *)
  (* HALT the program.                                                   *)
    if (@AddrIbsta      = NIL) Or
       (@AddrIberr      = NIL) Or
       (@AddrIbcntl     = NIL) Or
       (@DevClear       = NIL) Or
       (@FindLstn       = NIL) Or
       (@ReadStatusByte = NIL) Or
       (@SendIFC        = NIL) Or
       (@WaitSRQ        = NIL) Or
       (@ibclr          = NIL) Or
       (@ibdev          = NIL) Or
       (@ibonl          = NIL) Or
       (@ibrd           = NIL) Or
       (@ibwrt          = NIL)Then
       Begin
         str := 'GetProcAddresses FAILED!';
         MessageDlg(str, mtError, [mbOK], 0);
         (* Free the GPIB library.                                       *)
         FreeLibrary(Gpib32Lib);
         halt;
       End;

  (* Initialize GPIB global pointers to point to address location        *)
       Pibsta := @AddrIbsta;
       Piberr := @AddrIberr;
       Pibcntl := @AddrIbcntl;
  end; { LoadDLL }


(* =====================================================================
 *                       Procedure GPIBERR
 * This procedure will notify you that a NI-488.2 function failed by
 * printing an error message. The status variable IBSTA will be printed
 * in hexadecimal along with the mnemonic meaning of the bit position. The
 * status variable IBERR will be printed in decimal along with the
 * mnemonic meaning. The status variable will be printed in decimal.
 *
 * The NI-488 function ibonl is called to disable the hardware and
 * software.
 *
 * The HALT command stops execution of this program.
 * ===================================================================== *)
procedure gpiberr(msg: string);
  var
    str       : string;      (* String used for displaying messages.     *)
    ibstaStr  : string;      (* String for IBSTA.                        *)
    iberrStr  : string;      (* String for IBERR.                        *)
    ibcntlStr : string;      (* String for IBCNTL.                       *)

  begin
    ibstaStr := IntToHex(Pibsta^, 4);
    iberrStr := IntToStr(Piberr^);
    str := msg + '  ibsta = $' + ibstaStr + '  iberr = ' + iberrStr;
    MessageDlg(str, mtError, [mbOK], 0);

    str := 'ibsta ===> ';
    if (Pibsta^ and ERR) <> 0 Then
      str := Concat(str, '  ERR');
    if (Pibsta^ and TIMO) <> 0 Then
      str := Concat(str, '  TMO');
    if (Pibsta^ and ENDgpib) <> 0 Then
      str := Concat(str, '  END');
    if (Pibsta^ and SRQI) <> 0 Then
      str := Concat(str, '  SRQI');
    if (Pibsta^ and RQS) <> 0 Then
      str := Concat(str, '  RQS');
    if (Pibsta^ and SPOLL) <> 0 Then
      str := Concat(str, '  SPOLL');
    if (Pibsta^ and EVENT) <> 0 Then
      str := Concat(str, '  EVENT');
    if (Pibsta^ and CMPL) <> 0 Then
      str := Concat(str, '  CMPL');
    if (Pibsta^ and LOK) <> 0 Then
      str := Concat(str, '  LOK');
    if (Pibsta^ and REM) <> 0 Then
      str := Concat(str, '  REM');
    if (Pibsta^ and CIC) <> 0 Then
      str := Concat(str, '  CIC');
    if (Pibsta^ and ATN) <> 0 Then
      str := Concat(str, '  ATN');
    if (Pibsta^ and TACS) <> 0 Then
      str := Concat(str, '  TACS');
    if (Pibsta^ and LACS) <> 0 Then
      str := Concat(str, '  LACS');
    if (Pibsta^ and DTAS) <> 0 Then
      str := Concat(str, '  DTAS');
    if (Pibsta^ and DCAS) <> 0 Then
      str := Concat(str, '  DCAS');

    MessageDlg(str, mtError, [mbOK], 0);

    str := 'iberr ===> ';
    if Piberr^ = EDVR Then
      str := Concat(str, '  EDVR');
    if Piberr^ = ECIC Then
      str := Concat(str, '  ECIC');
    if Piberr^ = ENOL Then
      str := Concat(str, '  ENOL');
    if Piberr^ = EADR Then
      str := Concat(str, '  EADR');
    if Piberr^ = EARG Then
      str := Concat(str, '  EARG');
    if Piberr^ = ESAC Then
      str := Concat(str, '  ESAC');
    if Piberr^ = EABO Then
      str := Concat(str, '  EABO');
    if Piberr^ = ENEB Then
      str := Concat(str, '  ENEB');
    if Piberr^ = EDMA Then
      str := Concat(str, '  EDMA');
    if Piberr^ = EOIP Then
      str := Concat(str, '  EOIP');
    if Piberr^ = ECAP Then
      str := Concat(str, '  ECAP');
    if Piberr^ = EFSO Then
      str := Concat(str, '  EFSO');
    if Piberr^ = EBUS Then
      str := Concat(str, '  EBUS');
    if Piberr^ = ESTB Then
      str := Concat(str, '  ESTB');
    if Piberr^ = ESRQ Then
      str := Concat(str, '  ESRQ');
    if Piberr^ = ETAB Then
      str := Concat(str, '  ETAB');

    MessageDlg(str, mtError, [mbOK], 0);

    ibcntlStr := IntToStr(Pibcntl^);
    str :=  'ibcntl = ' + ibcntlStr;
    MessageDlg(str, mtError, [mbOK], 0);

  (* Call the ibonl function to disable the hardware and software.       *)
    ibonl(Dev, 0);
  (* Initialize GPIB global pointers to point to address location        *)
    FreeLibrary(Gpib32Lib);

    halt;
end;


Procedure Tform1.send_message ( message_to_send: string );

begin
     ListBox1.Items.Add ( message_to_send );
     ListBox1.Refresh;
end;

(* ====================================================================
 *                   Procedure address_measure_subsystem
 * This procedure addresses the measurement subsystem but does nothing else
 * Just to make sure it is ready to receive other commands
 * ==================================================================== *)

procedure address_measure_subsystem;

var buf     : data_buffer;   (* R/W buffer.               *)

begin
    buf := 'HEADER OFF';     (* turn off header *)
    ibwrt(Dev, buf, strlen(buf));

     (*  Send WAVEFORM query to initialize waveform subsystem *)
    buf := 'MEASURE?';
    ibwrt(Dev, buf , strlen(buf));
    if (Pibsta^ AND ERR) <> 0 THEN GPIBerr('Measure command write error');
    ibrd(Dev, ValueStr, ARRAYSIZE);
    if (Pibsta^ AND ERR) <> 0 THEN GPIBerr('Measure command read error');
    SleepEx(command_delay, FALSE);     { delay of 100 ms to allow scope to complete command }
    waveform_subsystem_addressed := false;
    channel_subsystem_addressed := false;
    measure_subsystem_addressed := true;
end;


(* ====================================================================
 *                   Procedure address_waveform_subsystem
 * This procedure addresses the waveform subsystem but does nothing else
 * Just to make sure it is ready to receive other commands
 * ==================================================================== *)

procedure address_waveform_subsystem;

var buf     : data_buffer;   (* R/W buffer.               *)

begin
    buf := 'HEADER OFF';     (* turn off header *)
    ibwrt(Dev, buf, strlen(buf));

     (*  Send WAVEFORM query to initialize waveform subsystem *)
    buf := 'WAVEFORM?';
    ibwrt(Dev, buf , strlen(buf));
    ibrd(Dev, ValueStr, ARRAYSIZE);
    waveform_subsystem_addressed := true;
    channel_subsystem_addressed := false;
    measure_subsystem_addressed := false;
end;




(* =====================================================================
 *                        Procedure Display_graph
 * This procedure draws a graph in box using the current graph data.
 * Linked to PaintBox1 in Form1 ONCLICK.
 *
 * ===================================================================== *)

Procedure TForm1.Display_graph ( Sender: TObject );

const td = 5;  (* top displacement in %, (for legend, title *)

var logscale: boolean;
    number_of_graphs: integer;
    layout: integer;                { which screen layout to use 1-5 }
    y_min_mixed, y_max_mixed: real;
    i: integer;
    graph_title_string: string;

(* ==================Subprocedures to do basic screen layout================= *)

Procedure layout_channel_1;


begin
      case layout of
           1: xystartgraph (0, 100, td, 100,      {use entire paint box}
                            40, 0, 0, 40,        {offset for axes }
                            clipoff );
           2: xystartgraph (0, 100, td, 50,       {top half of graph }
                            40, 0, 0, 40,
                            clipoff );
           3: xystartgraph (0, 50, td, 50,
                            40, 0, 0, 40,
                            clipoff );                  { upper left }
           4: xystartgraph (0, 50, td, 50,
                            40, 0, 0, 40,
                            clipoff );                   { upper left }
           5: xystartgraph (0, 50, td, 100,
                            40, 0, 0, 40,
                            clipoff );                   {left half }
           end;
end; { layout_channel_1 }



Procedure layout_channel_2;

begin
      case layout of
           1: xystartgraph (0, 100, td, 100,      {use entire paint box}
                            40, 0, 0, 40,        {offset for axes }
                            clipoff );
           2: if channel_1_display
                 then xystartgraph (0, 100, 50+td, 100,       {use bottom half of graph }
                                    40, 0, 0, 40,
                                    clipoff )
                 else xystartgraph (0, 100, td, 50,       {use top half of graph }
                                    40, 0, 0, 40,
                                    clipoff );
           3: xystartgraph (50, 100, td, 100,
                            40, 0, 0, 40,
                            clipoff );                  { right half }
           4: xystartgraph (50, 100, td, 50,
                            40, 0, 0, 40,
                            clipoff );                   { upper right }
           5: xystartgraph (50, 100, td, 50,
                            40, 0, 0, 40,
                            clipoff );                   {left half }
           end;
end; { Layout_channel_2 }

Procedure layout_fft_1;

begin
      case layout of
           1: xystartgraph (0, 100, td, 100,      {use entire paint box}
                            40, 0, 0, 40,        {offset for axes }
                            clipoff );
           2: xystartgraph (0, 100, 50+td, 100,       {use bottom half of graph }
                                    40, 0, 0, 40,
                                    clipoff );

           3: xystartgraph (0, 50, 50+td, 100,
                            40, 0, 0, 40,
                            clipoff );                  { lower left }
           4: xystartgraph (0, 50, 50+td, 100,
                            40, 0, 0, 40,
                            clipoff );                   { lower left }
           5: xystartgraph (0, 50, td, 100,
                            40, 0, 0, 40,
                            clipoff );                   {left half }
           end;
end; { Layout_fft_1 }

Procedure layout_fft_2;

begin
      case layout of
           1: xystartgraph (0, 100, td, 100,      {use entire paint box}
                            40, 0, 0, 40,        {offset for axes }
                            clipoff );
           2: xystartgraph (0, 100, 50+td, 100,       {use bottom half of graph }
                                    40, 0, 0, 40,
                                    clipoff );

           3: xystartgraph (50, 100, td, 100,
                            40, 0, 0, 40,
                            clipoff );                  { right half }
           4: xystartgraph (50, 100, 50+td, 100,
                            40, 0, 0, 40,
                            clipoff );                   { lower right }
           5: xystartgraph (50, 100, 50+td, 100,
                            40, 0, 0, 40,
                            clipoff );                   {lower right }
           end;
end; { subprocedure layout_fft_2 }




(* =================== Subprocedure to plot a waveform ===================== *)

Procedure plot_waveform_graph ( channel_number: integer; axis_display: boolean );

var i: integer;
    y_min_to_use, y_max_to_use: real;
    time_scale: string;
    adjustment: real;
    show_zero: boolean;
    ok: boolean;
    y_actual_range: real;
    y_display_range_ratio: real;
    y_zero_offset: real;

begin
   show_zero := true;
   with waveform_data[channel_number] do
        begin
                 (* First make a suitable time scale *)
        if x_max >= 1.1
           then begin
                time_scale := 'time (seconds)';
                multiplier := 1;
                end;
        if (x_max < 1.1) AND (x_max > 1.1e-3)
           then begin
                time_scale := 'time (milliseconds)';
                multiplier := 1000;
                end;
        if (x_max <= 1.1e-3) and (x_max > 1.1e-6)
           then begin
                time_scale := 'time (microseconds)';
                multiplier := 1e6;
                end;
        if (x_max < 1.1e-6)
           then begin
                time_scale := 'time (nanoseconds)';
                multiplier := 1e9;
                end;

        if axis_display then
           begin            (* Setup axes *)
           xyxaxis ( clblack,                 { x axis color }
                     x_min * multiplier,
                     x_max * multiplier,      { starting and ending time points }
                     0,                       { increment (not applicable) }
                     0,                       { zoom (not applicable) }
                     time_scale,                  { set to units, e.g., msec }
                     gridon,                  { display grid }
                     logscale,                { off }
                     false );                 { let graph routine figure out notation }

                         (* Now figure out y display axis values *)
           if y_min_actual*y_max_actual < 0 then ok := true else ok := false;
           if not ok then  (* do we want to display zero line? *)
              begin
              y_actual_range := abs(y_max_actual-y_min_actual);
              y_zero_offset := 0.05 * y_actual_range;     { amount to offset zero line }
              y_display_range_ratio := min(abs(y_min_actual), abs(y_max_actual))/ y_actual_range;
              if y_display_range_ratio > 1 then show_zero := false;
              end;

                     (* First do max side -- make sure there is enough
                        space between waveform and top edge of screen *)
           if y_max_actual > 0
              then adjustment := adjustment_factor
              else adjustment := 1/adjustment_factor;

           if y_max_clipped then y_max_to_use := y_max_actual*adjustment
                            else y_max_to_use := min ( y_max, y_max_actual*adjustment );

                     (* Now do bottom side *)
           if y_min_actual > 0
              then adjustment := 1/adjustment_factor
              else adjustment := adjustment_factor;

           if y_min_clipped then y_min_to_use := y_min_actual*adjustment
                            else y_min_to_use := max ( y_min, y_min_actual*adjustment );

           if not ok and show_zero then   (* force display of zero line *)
              if y_min_actual < 0
                 then y_max_to_use := 0 {y_zero_offset}
                 else y_min_to_use := 0 {-y_zero_offset};

           xyyaxis ( clblack,                 { x axis color }
                     y_min_to_use,
                     y_max_to_use,            { ordinate min, max values }
                     0,                       { increment (not applicable) }
                     0,                       { zoom (not applicable) }
                     'voltage',               { later set to units, e.g., mv }
                     1,                       { outside left }
                     gridon,                  { display grid }
                     logscale,                { off }
                     false );                 { for now use scientific notation }
           end;
           Paintbox1.Canvas.Pen.Width := 2;   { set line width }
           case channel_number of
                1: paintbox1.Canvas.Pen.Color := clred;
                2: paintbox1.Canvas.Pen.Color := clgreen;
                end;

             (* Start drawing *)
        xymove ( data_points[1,1]*multiplier, data_points[2,1] );  { go to first point }
             (* Now draw the graph *)
        for i := 2 to points-1 do xydraw ( data_points[1,i]*multiplier, data_points[2,i] );

             (* Now draw a solid line representing the zero ordinate if showing *)
        if axis_display and (ok or (not ok and show_zero)) then
           begin
           paintbox1.Canvas.Pen.Color := clblack;
           xymove ( data_points[1,1]*multiplier, 0 );
           xydraw ( data_points[1,points-1]*multiplier, 0 );
           end;

        end; { with ... do }
end; { subprocedure Plot_waveform_graph }



(* =================== Subprocedure to plot an FFT ========================== *)

Procedure plot_FFT_graph ( channel_number: integer );

var i: integer;

begin
   with fft_data[ channel_number ] do
        begin
                    (* Setup axes *)
           xyxaxis ( clblack,                 { x axis color }
                     x_min, x_max,            { starting and ending time points }
                     0,                       { increment (not applicable) }
                     0,                       { zoom (not applicable) }
                     'frequency',                  { later set to units, e.g., msec }
                     gridon,                  { display grid }
                     logscale,                { off }
                     false );                 { for now use scientific notation }
           xyyaxis ( clblack,                 { x axis color }
                     y_min, y_max,            { starting and ending time points }
                     0,                       { increment (not applicable) }
                     0,                       { zoom (not applicable) }
                     'amplitude',               { later set to units, e.g., mv }
                     1,                       { outside left }
                     gridon,                  { display grid }
                     logscale,                { off }
                     false );                 { for now use scientific notation }
           paintbox1.Canvas.Pen.Width := 2;   { set line width }
           case channel_number of
                1: paintbox1.Canvas.Pen.Color := clblue;
                2: paintbox1.Canvas.Pen.Color := clpurple;
                end;

             (* Now draw the graph as a set of spaced bars *)
        for i := 1 to harmonics_to_display do
            begin
            xymove ( data_points[1,i], 0 );       { go to first point }
            xydraw ( data_points[1,i],data_points[2,i] );
            end;

        end; { with ... do }
end; { subprocedure Plot_FFT_graph }



(* =================== Main part of procedure Display_data ================= *)
begin
{  if paintbox_initialized then
     begin
                      (* First verify button settings *)
     with waveform_data[1] do if channel_1_first_display and channel_1_display then
        with RadioGroup5 do
             begin
               TRadioButton(components[0]).Checked := (coupling = 1);  (* AC *)
               TRadioButton(components[1]).Checked := (coupling = 2);  (* DC *)
               channel_1_first_display := false;
             end;
     with waveform_data[2] do if channel_2_first_display and channel_2_display then
        with RadioGroup7 do
             begin
               TRadioButton(components[0]).Checked := (coupling = 1);  (* AC *)
               TRadioButton(components[1]).Checked := (coupling = 2);  (* DC *)
               channel_2_first_display := false;
             end;
      end; (*if paintbox_initialized*)      }

          (* Get correct number of graphs *)
   number_of_graphs := 0;
   if mixed_display_mode > 0
      then number_of_graphs := 1     { this is a combined display so only 1 graph }
      else begin                     { separate graphs so figure out how many }
           if channel_1_display then number_of_graphs := number_of_graphs + 1;
           if channel_2_display then number_of_graphs := number_of_graphs + 1;
           if channel_1_FFT then number_of_graphs := number_of_graphs + 1;
           if channel_2_FFT then number_of_graphs := number_of_graphs + 1;
           end;

   logscale := false;
           (* Set background white, foreground black *)
   xycleargraph ( paintbox1, clwhite, clblack, 1.1 );

           (* Get layout of graphs on screen *)
   case number_of_graphs of
        1: layout := 1;    { whole screen }
        2: layout := 2;    { top half and bottom half }
        3: if (channel_1_display AND channel_1_FFT)
              then layout := 3     { left half split top & bottom, right half }
              else layout := 5;    { right half split top & bottom, left half }
        4: layout := 4;     { four equal sized quarters }
        end;

             { if mixed mode on then get ranges and plot }
   if mixed_display_mode > 0 then
      begin
      layout_channel_1;
      case mixed_display_mode of
           1: begin   { separate displays }
              y_min_mixed := min(waveform_data[1].y_min, waveform_data[2].y_min);
              y_max_mixed := max(waveform_data[1].y_max, waveform_data[2].y_max);
              for i := 1 to 2 do waveform_data[i].y_min := y_min_mixed;
              for i := 1 to 2 do waveform_data[i].y_max := y_max_mixed;
              y_min_mixed := min(waveform_data[1].y_min_actual, waveform_data[2].y_min_actual);
              y_max_mixed := max(waveform_data[1].y_max_actual, waveform_data[2].y_max_actual);
              for i := 1 to 2 do waveform_data[i].y_min_actual := y_min_mixed;
              for i := 1 to 2 do waveform_data[i].y_max_actual := y_max_mixed;
              waveform_data[1].y_max_clipped := (waveform_data[1].y_max_actual > waveform_data[1].y_max);
              waveform_data[1].y_min_clipped := (waveform_data[1].y_min_actual < waveform_data[1].y_min );

              plot_waveform_graph( 1, true );    { only do axis display for this one }
              plot_waveform_graph( 2, false );
              xytitle(clred,'Ch 1 & Ch 2');
              end;
           2: begin
              y_min_mixed := 1e50;
              y_max_mixed := -1e50;
              for i := 1 to points do
                  begin
                  waveform_data[1].data_points[2,i] :=
                       waveform_data[1].data_points[2,i] + waveform_data[2].data_points[2,i];
                  if waveform_data[1].data_points[2,i] > y_max_mixed
                     then y_max_mixed := waveform_data[1].data_points[2,i];
                  if waveform_data[1].data_points[2,i] < y_min_mixed
                     then y_min_mixed := waveform_data[1].data_points[2,i];
                  end;
              waveform_data[1].y_min_actual := y_min_mixed;
              waveform_data[1].y_max_actual := y_max_mixed;
              waveform_data[1].y_max_clipped := true;
              waveform_data[1].y_min_clipped := true;
              plot_waveform_graph( 1, true );
              xytitle ( clred, 'Ch 1 + Ch 2');
              end;
           3: begin
              y_min_mixed := 1e50;
              y_max_mixed := -1e50;
              for i := 1 to points do
                  begin
                  waveform_data[1].data_points[2,i] := waveform_data[1].data_points[2,i] - waveform_data[2].data_points[2,i];
                  if waveform_data[1].data_points[2,i] > y_max_mixed then y_max_mixed := waveform_data[1].data_points[2,i];
                  if waveform_data[1].data_points[2,i] < y_min_mixed then y_min_mixed := waveform_data[1].data_points[2,i];
                  end;
              waveform_data[1].y_min_actual := y_min_mixed;
              waveform_data[1].y_max_actual := y_max_mixed;
              waveform_data[1].y_max_clipped := true;
              waveform_data[1].y_min_clipped := true;
              plot_waveform_graph( 1, true );
              xytitle ( clred, 'Ch 1 - Ch 2');
              end;
           end; { case }
      end; { if mixed_mode_display > 0 }

             { mixed mode not on, so do one of these }
   if channel_1_display then
      begin
      layout_channel_1;
      plot_waveform_graph(1, true);
      with waveform_data[1] do if valid_measurement_data
           then graph_title_string := 'CHANNEL 1   ' +
                Convert_number_to_string (' [T=', period ) + 's' +
                Convert_number_to_string ('; f=', frequency ) + 'Hz' +
{                Convert_number_to_string ('; tr=', rise_time ) + 's' +     }
                Convert_number_to_string ('; Vrms=',rms_value ) + 'v ]'
           else graph_title_string := 'CHANNEL 1  [invalid measurement data]';
      xytitle(clred, graph_title_string);
      end;

   if channel_2_display then
      begin
      layout_channel_2;
      plot_waveform_graph(2, true);
      with waveform_data[2] do if valid_measurement_data
           then graph_title_string := 'CHANNEL 2   ' +
                Convert_number_to_string (' [T=', period ) + 's' +
                Convert_number_to_string ('; f=', frequency ) + 'Hz' +
{                Convert_number_to_string ('; tr=', rise_time ) + 's' +   }
                Convert_number_to_string ('; Vrms=',rms_value ) + 'v ]'
           else graph_title_string := 'CHANNEL 2  [invalid measurement data]';
      xytitle(clgreen, graph_title_string);
      end;

   if channel_1_FFT then
      begin
      layout_fft_1;
      plot_FFT_graph(1);
      xytitle(clblue, 'FFT for Ch 1');
      end;

   if channel_2_FFT then
      begin
      layout_fft_2;
      plot_FFT_graph(2);
      xytitle(clpurple, 'FFT for Ch 2');
      end;
    xyfinish;
    paintbox_initialized := true;
end;   { Tform1.Display_graph }


(* =======================================================================
 *                Procedure Output_Data_File
 * This procedure writes raw data out to a file for graphing or futher
 * analysis by other programs.  File is a standard text file
 *
 * ======================================================================= *)

Procedure Output_Data_File;

var  i,j: integer;
     output_data_file: textfile;  (* output file *)
     filename: string;
     waveforms: array[1..2] of boolean;
     ffts: array[1..2] of boolean;

begin
  waveforms[1] := channel_1_display;
  waveforms[2] := channel_2_display;
  ffts[1] := channel_1_fft;
  ffts[2] := channel_2_fft;

  for j := 1 to 2 do
      begin           (* Open output file for writing converted raw data for debugging *)
       if waveforms[j] then with waveform_data[j] do
          begin
            filename := 'c:\junk\waveform'+IntToStr(j)+'_data.dat';
            AssignFile ( output_data_file, filename );
            Rewrite ( output_data_file );
            Try
               writeln ( output_data_file,' Raw data ','      Time    ','   Voltage    ');
               For i := 1 to points do
               writeln ( output_data_file, raw_data[i]:8, '  ', data_points[1,i]:14,' ',
                   data_points[2,i]:14 );
               Finally CloseFile ( output_data_file );
            End; { try }
          end; { if waveform...begin }
       if ffts[j] then with fft_data[j] do
          begin
            filename := 'c:\junk\fft'+IntToStr(j)+'_data.dat';
            AssignFile ( output_data_file, filename );
            Rewrite ( output_data_file );
            Try
               writeln ( output_data_file,'   Frequency  ','   Amplitude  ');
               For i := 1 to points do
               writeln ( output_data_file, ' ', data_points[1,i],' ',
                   data_points[2,i] );
               Finally CloseFile ( output_data_file );
            End; { try }
          end; { if FFT ... begin }
      end; { for j.. }
      MessageDlg('Data files successfully written',mtInformation,[mbOK],0);
end; { Procedure Output_data_file }



(* =====================================================================
 *                       Procedure compute FFT
 * This procedure computes the FFT for the data displayed.  Data are filled
 * to make exactly 2048 points, as a power of 2 is required by the algorithm
 * Delta f = 1/(1024 * delta_x)
 *
 * ===================================================================== *)

Procedure compute_FFT ( channel_number: integer );

CONST
     eps = 1.0e-3;
     np = 1024;
     np2 = 1026;    {np + 2}
     pi = 3.14159265;

TYPE
    real = extended;
    gldarray = ARRAY [1..np2] of REAL;

VAR
   i,n: integer;
   data, size: gldarray;
   delta_f: real;
   FFT_data_array: data_point_array;
   dydx: real;                        { used for extrapolation to 1024 points }
   unit_time: real;
   big: real;                         { used for normalizing to largest value }
   max_time: real;                    { used for calculating correct frequency in output data }

(*$I FOUR1.PAS *)
(*$I REALFT.PAS *)

BEGIN
  harmonics_to_display := 40;
  with waveform_data [channel_number] do
       begin
       n := np div 2;
       unit_time := delta_x;
       max_time := unit_time*np;
       for i := 1 to points-1 do data[i] := data_points[2,i];  (* copy data to FFT array *)
                  { interpolate (actually extrapolate) to remaining points up to 1024
                     eliminated--leads to bizarre large values sometimes }
(*       dydx := (data_points[2,points-1]-data_points[2,points-3])/(2*delta_x);  *)
       for i := points to 1024 do data[i] := data[i-1] {+ dydx};
       realft(data,n,+1);
       delta_f := 1/(n*delta_x);
       big := 0;
       for i := 1 to n do
           begin
           size[i] := sqrt(sqr(data[2*i-1])+sqr(data[2*i]));
           FFT_data_array[1,i] := (i-1)/max_time;
           FFT_data_array[2,i] := size[i];
           if size[i] > big then big := size[i];
           end;
       end;

          (* Now copy the FFT data to the FFT graph record *)
  with fft_data [ channel_number ] do
       begin
       data_points := FFT_data_array;
                 { normalize with respect to total area }
       for i := 1 to 512 do data_points[2,i] := data_points[2,i]/big {area};
       for i := 513 to 1024 do      { by Nyquist criterion we only get 512 fft points }
           begin
           data_points[1,i] := 0;
           data_points[2,i] := 0;
           end;
       delta_x := delta_f;
       x_origin := 0;
       y_origin := 0;
       x_min := x_origin;
       x_max := harmonics_to_display*delta_x/2;
       y_min := 0;
       y_max := 1;
       line_color := 1;
       number_of_x_lines := 0;
       number_of_y_lines := 4;
       interpolation := false;
       end;
end;


(* =====================================================================
 *                       Procedure Convert_Data
 * This procedure converts raw data from HP scope to real numbers prior
 * to graphing.  Raw data is put into a record of form waveform_graph
 * Raw data is in form of integers with zero reference given.
 *
 * ===================================================================== *)

Procedure Convert_Data ( number_of_data_points: integer; channel_number: integer );

 var i,j: integer;
     integer_form: longint;
     number_of_times_to_convert: integer;

 Begin
   With waveform_data [channel_number] do
        begin
        if averaging then number_of_times_to_convert := number_to_average
                     else number_of_times_to_convert := 1;
        x_origin := xorigin;
        y_origin := yorigin;
        delta_x := xincrement;
        delta_y := yincrement;
        number_of_x_lines := 4;     (* not used *)
        number_of_y_lines := 4;     (* not used *)
        y_reference_zero := yreference;
        line_color := 1;

        if (coupling_parameter[1] = 'A') or (coupling_parameter[1] = '2')
           then coupling := 2
           else coupling := 1;

        data_points[1,0] := x_origin;
        data_points[2,0] := y_origin;
        y_average := 0;
                 { compute minima, maxima }
        x_min := x_origin;
        x_max := points*delta_x + x_origin;
        y_min := -y_reference_zero*delta_y;
        y_max := y_reference_zero*delta_y;

        y_min_actual := 1e50;
        y_max_actual := -1e-50;

                        { initialize accumulators }
        for i := 0 to 2002 do
            begin
            data_points[2,i] := 0;
            raw_data[i] := 0;
            end;
                 { convert raw integer format to correct real numbers }

        for i := 1 to number_of_data_points do
            begin
            data_points[1,i] := x_origin + i*delta_x;
            for j := 1 to number_of_times_to_convert do
                begin
                if data_in_byte_format then integer_form := byte_array[j,i]
                                       else integer_form := word_array[j,i];
                raw_data[i] := raw_data[i] + integer_form;
                data_points[2,i] := data_points[2,i] + (integer_form-y_reference_zero)*delta_y + y_origin;
                end;
                   (* convert to single frame *)
            if averaging then
               begin
               raw_data[i] := raw_data[i] div number_to_average;
               data_points[2,i] := data_points[2,i]/number_to_average;
               end;
                   (* do requested inversion *)
            if channel_inverted then data_points[2,i] := -data_points[2,i];
            y_average := y_average + data_points[2,i];
                  (* actual min & max may differ from those calculated with y_reference_zero *)
            y_max_actual := max(data_points[2,i], y_max_actual);
            y_min_actual := min(data_points[2,i], y_max_actual);
            end;  { for i:=1 to number...}
         y_average := y_average/points;
         y_max_clipped := (y_max_actual > y_max);
         y_min_clipped := (y_min_actual < y_min );
         end;  { With }
 End;  { Convert Data }


(* =====================================================================
 *              Procedure Get_Waveform_Measurement_Data
 * This procedure gets basic measured data about waveform from HP scope
 * and puts into waveform_data corresponding to channel
 *
 * ===================================================================== *)

Procedure get_waveform_measurement_data ( channel_number: integer );

var buffer  : string[200];   (* Assigned the value of the READING.       *)
    i: integer;
    measurement_data_validity_status: integer;
    error_index: integer;
    error_msg: string;
    command_delay: integer;

begin
  command_delay := 40;        (* 40 ms delay *)
            (* Now get period, frequency, rise, fall times, rms value *)
  address_measure_subsystem;             (* turn it on *)
  if channel_number = 1                  (* select channel *)
     then Vstr := 'SOURCE GRAPH 1'
     else Vstr := 'SOURCE GRAPH 2';
  ibwrt(Dev, Vstr , strlen(Vstr));
  if (Pibsta^ AND ERR) <> 0 THEN GPIBerr('Measurement system write error');
  SleepEx(40, FALSE);     { delay of 100 ms to allow scope to complete command }

                   (* See if data valid *)

  Vstr := 'VALID ?';
  ibwrt(Dev, Vstr , strlen(Vstr));
  ibrd(Dev, ValueStr, ARRAYSIZE);
  SleepEx(command_delay, FALSE);     { delay of 100 ms to allow scope to complete command }
  ValueStr[Pibcntl^ - 1] := #0;
  FOR i := 1 to Pibcntl^ DO buffer[i] := ValueStr[i-1];
  SetLength (buffer, Pibcntl^-2);       (* get rid of trailing CRLF *)
  val ( buffer, measurement_data_validity_status, error_index );
  if measurement_data_validity_status <> 1
     then begin             (* some problem, e.g., clipped *)
         case measurement_data_validity_status of
              0:  error_msg := 'no waveform';
             -1:  error_msg := 'scaling';
             -2:  error_msg := 'threshold';
             -3:  error_msg := 'clipped';
             -6:  error_msg := 'not found';
            end;
         msg := 'Waveform measurement data invalid, code = ' + error_msg;
         waveform_data [ channel_number ].valid_measurement_data := false;
         {         send_message ( msg );}
         end
    else with waveform_data [ channel_number ] do
         begin
         valid_measurement_data := true;
{         Vstr := 'RISE ?';
         ibwrt(Dev, Vstr , strlen(Vstr));
         ibrd(Dev, ValueStr, ARRAYSIZE);
         SleepEx(command_delay, FALSE);     (* delay of 100 ms to allow scope to complete command *)
         ValueStr[Pibcntl^ - 1] := #0;
         FOR i := 1 to Pibcntl^ DO buffer[i] := ValueStr[i-1];
         SetLength (buffer, Pibcntl^-2);       (* get rid of trailing CRLF *)
         val ( buffer, rise_time, error_index );                                   }

         Vstr := 'PERIOD ?';
         ibwrt(Dev, Vstr , strlen(Vstr));
         ibrd(Dev, ValueStr, ARRAYSIZE);
         SleepEx(20, FALSE);     { delay of 100 ms to allow scope to complete command }
         ValueStr[Pibcntl^ - 1] := #0;
         FOR i := 1 to Pibcntl^ DO buffer[i] := ValueStr[i-1];
         SetLength (buffer, Pibcntl^-2);       (* get rid of trailing CRLF *)
         val ( buffer, period, error_index );
         frequency := 1/period;

{         Vstr := 'FREQUENCY ?';
         ibwrt(Dev, Vstr , strlen(Vstr));
         ibrd(Dev, ValueStr, ARRAYSIZE);
         SleepEx(command_delay, FALSE);     (* delay of 100 ms to allow scope to complete command *)
         ValueStr[Pibcntl^ - 1] := #0;
         FOR i := 1 to Pibcntl^ DO buffer[i] := ValueStr[i-1];
         SetLength (buffer, Pibcntl^-2);       (* get rid of trailing CRLF *)
         val ( buffer, frequency, error_index );    }

         Vstr := 'VRMS ?';
         ibwrt(Dev, Vstr , strlen(Vstr));
         ibrd(Dev, ValueStr, ARRAYSIZE);
         SleepEx(20, FALSE);     { delay of 100 ms to allow scope to complete command }
         ValueStr[Pibcntl^ - 1] := #0;
         FOR i := 1 to Pibcntl^ DO buffer[i] := ValueStr[i-1];
         SetLength (buffer, Pibcntl^-2);       (* get rid of trailing CRLF *)
         val ( buffer, rms_value, error_index );
         end;

end; {get_waveform_measurement_data}


(**************** Procedure to read and write graph data  ******************
 * This procedure reads a block of graph data from the scope and requests its
 * display on the computer screen
 ============================================================================ *)

Procedure get_waveform_data ( channel_number: integer );

var
  i: integer;
  number_sign,block_name: char;
  block_length: longint;
  block_length_word: word;
  buf     : data_buffer;   (* R/W buffer.               *)
  number_of_times_to_read: integer;

begin
    with waveform_data [ channel_number ] do
       if averaging
          then number_of_times_to_read := number_to_average
          else number_of_times_to_read := 1;

    for i := 1 to number_of_times_to_read do
        begin
            (* Get a block of waveform data *)
        buf := 'DATA?';
        ibwrt(Dev, buf , strlen(buf));
        if (Pibsta^ AND ERR) <> 0
           THEN GPIBerr('Error writing DATA query')
           ELSE begin

                    (* Get block header items *)
                ibrd (Dev, number_sign, 1);           (* first get #A *)
                if (Pibsta^ AND ERR) <> 0 THEN GPIBerr('Error reading number sign');
                ibrd (Dev, block_name, 1);
                if (Pibsta^ AND ERR) <> 0 THEN GPIBerr('Error reading block name');
                ibrd (Dev, block_length_word, 1);     (* 16 bit word *)
                if (Pibsta^ AND ERR) <> 0 THEN GPIBerr('Error reading block length');

                block_length := block_length_word;    (* convert to integer *)

             (* check for glitch in block length *)
{  if block_length <> points then
     begin
     msg := 'Mismatch between block length ('+inttostr(block_length) +
            ') and points (' + inttostr(points) +')';
     ListBox1.Items.Add(msg);
     ListBox1.Refresh;
     end;      }

                 (* Now try to read the actual quantized data
                    values are integers from 0-32000 presumably *)
                if data_in_byte_format then ibrd ( Dev, byte_array[i], points )
                                       else ibrd (Dev, word_array[i], 2*points);
                if (Pibsta^ AND ERR) <> 0
                   THEN GPIBerr('Unable to read data');
                end; { if (pibsta^ AND error }
         SleepEx(command_delay, FALSE);     { delay of 100 ms to allow scope to complete command }
         end; { for i := number... }
         convert_data ( points, channel_number );
         compute_FFT ( channel_number );
         get_waveform_measurement_data ( channel_number );

end; { procedure get_waveform_data }



(* =====================================================================
 *                    Procedure TForm1.Button1Click
 * This procedure is activated when the INITIALIZE button is clicked. This
 * procedure looks for the HP54200A. If the HP54200A is a
 * listener on the GPIB, it reads the WAVEFORM data.
 * ===================================================================== *)

procedure TForm1.initialize_hp54200A;

var
    buffer  : string[100];   (* Assigned the value of the READING.       *)
    i       : integer;       (* loop counter *)
    debug_data: boolean;


        (* ***************** Begin Main Part of Procedure ****************** *)

  begin

 (* Load the GPIB-32.DLL Library.                                         *)
    loadDLL;

 (* Clear the List Box screen.                                           *)
    ListBox1.Clear;

  (* Your board needs to be the Controller-In-Charge in order to find    *)
  (* all listeners on the GPIB. To accomplish this, the function         *)
  (* SendIFC is called. If the error bit ERR is set in IBSTA, call       *)
  (* GPIBERR with an error message.                                      *)
{    SendIFC(0);
    if (Pibsta^ AND ERR) <> 0 THEN gpiberr('SendIFC error');  }

    (*
      * The application brings the oscilloscope online using ibdev. A
      * device handle, Dev, is returned and is used in all subsequent
      * calls to the device.
      *)
            (* First do some initialization *)
     debug_data := false;
     channel_1_display := false;
     channel_2_display := false;
     channel_1_invert := false;
     channel_2_invert := false;
     channel_sum := false;
     channel_1_FFT := false;
     channel_2_FFT := false;
     for i := 1 to 2 do with waveform_data[i] do
         begin
         channel_inverted := false;
         averaging := false;
         end;
            (* Send out starting message to status display *)
     msg := 'Bringing scope online and initializing...';
     ListBox1.Items.Add(msg);
     ListBox1.Refresh;

     if debug_data then
        begin
        msg := 'Attempting to bring scope online and get device handle... ';
        ListBox1.Items.Add(msg);
        ListBox1.Refresh;
        end;
     Dev := ibdev(BDINDEX, PRIMARY_ADDR_OF_SCOPE,
                  NO_SECONDARY_ADDR, TIMEOUT, EOTMODE, EOSMODE);
     if (Pibsta^ AND ERR) <> 0 Then
        GPIBerr('Unable to open device');

     if debug_data then
        begin
        msg := 'Scope handle = ' + InttoStr( Dev );
        ListBox1.Items.Add(msg);
        ListBox1.Refresh;
        end;

     (*
      * The application resets the GPIB portion of the oscilloscope by
      * calling ibclr.
      *)
     ibclr(Dev);
     if (Pibsta^ AND ERR) <> 0 Then
        GPIBerr('Unable to clear device');

     (*
      * The application writes the text in CommandBox to the
      * oscilloscope.
      *)
     if debug_data then
        begin
        msg := 'Attempting to get scope id... ';
        ListBox1.Items.Add(msg);
        ListBox1.Refresh;
        end;

     Vstr := 'ID ?';
     ibwrt(Dev, Vstr , strlen(Vstr));
     if (Pibsta^ AND ERR) <> 0 THEN
        GPIBerr('Unable to write to device');

     (*
      * The application reads the ASCII string from the oscilloscope
      * into the ValueStr variable.
      *)
     ibrd(Dev, ValueStr, ARRAYSIZE);
     if (Pibsta^ AND ERR) <> 0 THEN
        GPIBerr('Unable to read from device');

     (*
      * Assume that the returned string contains ASCII data. NULL
      * terminate the string using the value in ibcntl which is the
      * number of bytes read in.
      *)
     ValueStr[Pibcntl^ - 1] := #0;

     (* Assign the array ValueStr to the string BUFFER.          *)
     (* Print the name identification of the device.             *)
      FOR i := 1 to Pibcntl^ DO buffer[i] := ValueStr[i-1];
      SetLength (buffer, Pibcntl^-2);       (* get rid of trailing CRLF *)
      if debug_data then
         begin
         msg := 'The instrument response is: ' + buffer;
         ListBox1.Items.Add(msg);
         ListBox1.Refresh;
         msg := 'The length of the response is ' + IntToStr(Pibcntl^) + 'characters';
         ListBox1.Items.Add(msg);
         ListBox1.Refresh;
         end;

      (* Determine if the name identification is the HP54200A. If it is *)
      (* the HP54200A, assign PAD to the HP542000A, print message that  *)
      (* HP54200A has been found. If HP54200A was not found, then       *)
      (* print message that it was not found and terminate the program. *)
     if buffer = '"HP54200A"'    (* comes back quoted *)
        then begin
             if debug_data then
                begin
                msg := '***** We found the HP54200A ***** ';
                ListBox1.Items.Add(msg);
                ListBox1.Refresh;
                end;
             end     (* The reading from the oscilloscope is displayed in the List box. *)
        else begin
             msg := 'Did not find the HP54200A!';
             MessageDlg(msg, mtError, [mbOK], 0);

           (* Call the ibonl function to disable the hardware and software.    *)
             ibonl(DEV, 0);
           (* Free the GPIB library.                                           *)
             FreeLibrary(Gpib32Lib);
             halt;
             end;
    msg := 'Initialization successful';
    ListBox1.Items.Add(msg);
    ListBox1.Refresh;
    Initialized := true;
end;  { TForm1.Button1Click }



(* ===========================================================================
 *                    Procedure Get_Information_About_Waveform
 *  Use the subroutine ibwrt to send device configuration commands to the
 *  HP54200A.  Instruct it to send back PREAMBLE. Copy the string into
 *  the command array.  If the error bit ERR is set, call GPIBERR with an
 *  error message. Ref HP manual p. 8-129.
 * =========================================================================== *)

Procedure TForm1.Get_Information_About_Waveform ( Waveform_Number: integer );

Var
    buf     : data_buffer;   (* R/W buffer.               *)
    i       : integer;       (* loop counter *)
    preamble_string: string;
    debug_data: boolean;
    phrase_buffer        :       string;       { holds text to be parsed }



(* =============================================================================
 * Subprocedure to parse HP54200A preamble and extract info
 * Items in preamble are comma delimited
 * ===========================================================================*)

                    { send string to be parsed }
PROCEDURE parse_preamble ( preamble_true: boolean; preamble: string; string_length: integer );

VAR character_pointer    :       integer;      { current position in string }
    word_found           :       boolean;
    word_length          :       integer;
    i                    :       integer;      { loop counter }
    error_index          :       integer;      { set to problem position or 0 if OK }
    debug                :       boolean;      { true if parsed info to be printed }
    end_character        :       char;         { denotes end of item }


            { This subsubprocedure gets a word out of the preamble string and puts
              it into BUFFER }

  PROCEDURE read_a_word (var ok: boolean; var phrase_buffer_length: INTEGER);

  VAR i: integer;

  BEGIN
    phrase_buffer := '                         ';        { blank it out }
    i := 0;
    ok := true;
    WHILE (preamble [character_pointer] <> end_character) AND ok DO
          BEGIN
          i := i+1;
          phrase_buffer[i] := preamble [character_pointer];
          character_pointer := character_pointer + 1;
          if character_pointer > string_length THEN ok := false;
          END;
 {                msg := 'phrase = ' + phrase_buffer + '; length = ' + IntToStr(i);
                ListBox1.Items.Add(msg);
                ListBox1.Refresh;           }
    if preamble_true then character_pointer := character_pointer + 1    { go past comma }
                     else character_pointer := character_pointer + 2;   { go past CRLF }
     phrase_buffer_length := i;
     SetLength ( phrase_buffer, i );
  END;    { read_a_word }


BEGIN   { main part of Parse_Preamble subprocedure }
{  msg := 'Parsing preamble...';
  ListBox1.Items.Add(msg);
  ListBox1.Refresh; }

  debug := FALSE;
  IF preamble_true THEN end_character := ','
                   ELSE end_character := chr(13);    { CR for WAVEFORM parsing }
  Character_pointer := 1;

  IF NOT preamble_true THEN    { Only use this code if WAVEFORM being parsed }
     BEGIN
           { source is graph or memory; gives source of graph data read from HP54200A }
      graph_source := '       ';
      read_a_word ( word_found, word_length );
      for i := 1 to word_length do graph_source[i] := phrase_buffer[i];
      if debug then begin
                    msg := 'Graph source = ' + graph_source;
                    ListBox1.Items.Add(msg);
                    ListBox1.Refresh;
                    end;

            { VALID checks for graph data OK }
      valid_data := '          ';
      read_a_word ( word_found, word_length );
      for i := 1 to word_length do valid_data[i] := phrase_buffer[i];
      if debug then begin
                    msg := 'Valid = ' + valid_data;
                    ListBox1.Items.Add(msg);
                    ListBox1.Refresh;
                    end;
     END; { if NOT preamble_true }

         { format must be byte (1) or word (2); gives format of graph data read from HP54200A }
  format_parameter := '    ';
  read_a_word ( word_found, word_length );
  for i := 1 to word_length do format_parameter[i] := phrase_buffer[i];
  if format_parameter = '1' then data_in_byte_format := true
                            else data_in_byte_format := false;
  if debug then begin
                msg := 'Format parameter = ' + format_parameter;
                ListBox1.Items.Add(msg);
                ListBox1.Refresh;
                end;

         { type is normal, average, or envelope value }
  type_parameter := '        ';
  read_a_word ( word_found, word_length );
  for i := 1 to word_length do type_parameter[i] := phrase_buffer[i];
  if debug then begin
                msg := 'type parameter = ' + type_parameter;
                ListBox1.Items.Add(msg);
                ListBox1.Refresh;
                end;

         { number of points in graph }
  read_a_word ( word_found, word_length );
  val ( phrase_buffer, points, error_index );
  if error_index <> 0 then begin
                           msg := 'error decoding points, string = '
                           + phrase_buffer + ' problem at position '
                           + IntToStr(error_index);
                           ListBox1.Items.Add(msg);
                           ListBox1.Refresh;
                           end
                      else if debug then
                           begin
                           msg := 'number of points = ' + IntToStr(points);
                           ListBox1.Items.Add(msg);
                           ListBox1.Refresh;
                           end;

           { number of frames averaged or frame number }
  read_a_word ( word_found, word_length );
  val ( phrase_buffer, frame_count, error_index );
  if error_index <> 0 then begin
                           msg := 'error decoding frame_count, string = '
                           + phrase_buffer + ' problem at position '
                           + IntToStr(error_index);
                           ListBox1.Items.Add(msg);
                           ListBox1.Refresh;
                           end
                      else if debug then
                           begin
                           msg := 'frame_count = ' + IntToStr(frame_count);
                           ListBox1.Items.Add(msg);
                           ListBox1.Refresh;
                           end;

             { Size of delta x }
  read_a_word ( word_found, word_length );
  val ( phrase_buffer, xincrement, error_index );
  if error_index <> 0 then begin
                           msg := 'error decoding xincrement, string = '
                           + phrase_buffer + ' problem at position '
                           + IntToStr(error_index);
                           ListBox1.Items.Add(msg);
                           ListBox1.Refresh;
                           end
                      else if debug then
                           begin
                           msg := 'xincrement = ' + FloatToStr(xincrement);
                           ListBox1.Items.Add(msg);
                           ListBox1.Refresh;
                           end;

            { value of x (time) origin }
  read_a_word ( word_found, word_length );
  val ( phrase_buffer, xorigin, error_index );
  if error_index <> 0 then begin
                           msg := 'error decoding xorigin, string = '
                           + phrase_buffer + ' problem at position '
                           + IntToStr(error_index);
                           ListBox1.Items.Add(msg);
                           ListBox1.Refresh;
                           end
                      else if debug then
                           begin
                           msg := 'xorigin = ' + FloatToStr(xorigin);
                           ListBox1.Items.Add(msg);
                           ListBox1.Refresh;
                           end;

            { number of abscissa lines in graph }
  read_a_word ( word_found, word_length );
  val ( phrase_buffer, xreference, error_index );
  if error_index <> 0 then begin
                           msg := 'error decoding xreference, string = '
                           + phrase_buffer + ' problem at position '
                           + IntToStr(error_index);
                           ListBox1.Items.Add(msg);
                           ListBox1.Refresh;
                           end
                      else if debug then
                           begin
                           msg := 'xreference = ' + IntToStr(xreference);
                           ListBox1.Items.Add(msg);
                           ListBox1.Refresh;
                           end;

            { value of delta y }
  read_a_word ( word_found, word_length );
  val ( phrase_buffer, yincrement, error_index );
  if error_index <> 0 then begin
                           msg := 'error decoding yincrement, string = '
                           + phrase_buffer + ' problem at position '
                           + IntToStr(error_index);
                           ListBox1.Items.Add(msg);
                           ListBox1.Refresh;
                           end
                      else if debug then
                           begin
                           msg := 'yincrement = ' + FloatToStr(yincrement);
                           ListBox1.Items.Add(msg);
                           ListBox1.Refresh;
                           end;

           { value of y origin }
  read_a_word ( word_found, word_length );
  val ( phrase_buffer, yorigin, error_index );
  if error_index <> 0 then begin
                           msg := 'error decoding yorigin, string = '
                           + phrase_buffer + ' problem at position '
                           + IntToStr(error_index);
                           ListBox1.Items.Add(msg);
                           ListBox1.Refresh;
                           end
                      else if debug then
                           begin
                           msg := 'yorigin = ' + FloatToStr(yorigin);
                           ListBox1.Items.Add(msg);
                           ListBox1.Refresh;
                           end;

            { number of ordinate lines on graph }
  read_a_word ( word_found, word_length );
  val ( phrase_buffer, yreference, error_index );
  if error_index <> 0 then begin
                           msg := 'error decoding yreference, string = '
                           + phrase_buffer + ' problem at position '
                           + IntToStr(error_index);
                           ListBox1.Items.Add(msg);
                           ListBox1.Refresh;
                           end
                      else if debug then
                           begin
                           msg := 'yreference = ' + IntToStr(yreference);
                           ListBox1.Items.Add(msg);
                           ListBox1.Refresh;
                           end;

  coupling_parameter := '  ';
  read_a_word ( word_found, word_length );
  for i := 1 to word_length do coupling_parameter[i] := phrase_buffer[i];
  if debug then begin
                msg := 'coupling parameter = ' + coupling_parameter;
                ListBox1.Items.Add(msg);
                ListBox1.Refresh;
                end;

  waveform_label := '           ';
  read_a_word ( word_found, word_length );
           (* Use HP spec of 10 chars for this item; word_length generates
             range check error for some peculiar reason *)
  for i := 1 to 10 {word_length} do waveform_label[i] := phrase_buffer[i];
  if debug then begin
                msg := 'waveform label = ' + waveform_label;
                ListBox1.Items.Add(msg);
                ListBox1.Refresh;
                end;

END;  { Parse_Preamble }



Begin { main part of procedure }
    debug_data := false;
    buf := 'HEADER OFF';
    ibwrt(Dev, buf, strlen(buf));
    if (Pibsta^ AND ERR) <> 0 THEN GPIBerr('Error writing HEADER OFF command');
    if not waveform_subsystem_addressed then address_waveform_subsystem;

            (* First set the correct source *)
    if waveform_number = 1 then buf := 'SOURCE CHANNEL1'
                           else buf := 'SOURCE CHANNEL2';
    ibwrt ( Dev, buf, strlen(buf) );
    SleepEx ( command_delay, FALSE );

     (*  Send WAVEFORM query to get info about selected waveform,
      *  and if successful, parse results *)
    buf := 'WAVEFORM?';
    ibwrt(Dev, buf , strlen(buf));
    if (Pibsta^ AND ERR) <> 0
       THEN GPIBerr('Error writing PREAMBLE query')
       ELSE BEGIN
            waveform_subsystem_addressed := true;
            if debug_data then
               begin
               msg := 'Getting ready to read ' + buf;
               ListBox1.Items.Add(msg);
               ListBox1.Refresh;
               end;
            ibrd(Dev, ValueStr, ARRAYSIZE);
            if (Pibsta^ AND ERR) <> 0
               THEN GPIBerr('Unable to read ' + buf)
               ELSE BEGIN
                   Preamble_string := ValueStr;
                   if debug_data then
                       begin
                       msg := 'Getting ready to parse ' + buf + ' string:';
                       ListBox1.Items.Add(msg);
                       ListBox1.Refresh;
                       ListBox1.Items.Add(preamble_string);
                       ListBox1.Refresh;
                       end;
                    parse_preamble ( FALSE, Preamble_string, Length(Preamble_string) );
                    END;
            END;
End;  { Procedure Get_Information_About_Waveform }



(* ==================== Start Button Handling Code ========================*)


(* =====================================================================
 *                    Procedure TForm1.Button1Click
 * This procedure is activated when the REFRESH button is clicked. Currently
 * displayed data are refreshed with new data from scope.
 * ===================================================================== *)

procedure TForm1.Button1Click(Sender: TObject);

var i: integer;

begin
  if mixed_display_mode > 0
     then for i := 1 to 2 do
              begin
              Get_information_about_Waveform (i);
              Get_Waveform_Data(i);
              end
     else begin
          if (channel_1_display OR channel_1_fft) then
             begin
             Get_Information_About_Waveform (1);
             Get_Waveform_Data (1);
             end;
         if (channel_2_display OR channel_2_fft) then
            begin
            Get_Information_About_Waveform (2);
            Get_Waveform_Data (2);
            end;
         end;
  paintbox1.refresh;
end; {TForm1.Button1Click}



(* =====================================================================
 *                    Procedure TForm1.Button2Click
 * This procedure is activated when the QUIT button is clicked. This
 * procedure terminates the program.
 * ===================================================================== *)
procedure TForm1.Button2Click(Sender: TObject);

var  buf     : data_buffer;   (* R/W buffer.               *)

begin
   if not waveform_subsystem_addressed then
      begin
      address_waveform_subsystem;
          (* Reset source to Channel 1 *)
      buf := 'SOURCE CHANNEL1';
     ibwrt ( Dev, buf, strlen(buf) );
     end;

   (* If GPIB library is loaded, then free it before exiting the program. *)
   if Gpib32Lib <> 0 then begin
     FreeLibrary(Gpib32Lib);
   end;

   Close;
end;



(* =====================================================================
 *                    Procedure TForm1.Button3Click
 * This procedure is activated when the ABOUT button is clicked. This
 * procedure displays a pop up message box with information stating
 * that this program is a GPIB sample program.
 * ===================================================================== *)
procedure TForm1.Button3Click(Sender: TObject);
begin
  MessageDlg('HP54200A GPIB NI-488.2 Display & Control Program',mtInformation,[mbOK],0);
end;




(* =====================================================================
 *                    Procedure TForm1.Button5Click
 * This procedure is activated when the AUTOSCALE button is clicked. This
 * procedure causes scope to rescale currently displayed waveforms.
 * ===================================================================== *)
procedure TForm1.Button5Click(Sender: TObject);

var buf     : data_buffer;   (* R/W buffer.               *)
    channel_number: integer;

begin
  if not initialized then initialize_hp54200A;        { make sure initialization complete }
  buf := 'AUTOSCALE FULL';
  ibwrt(Dev, buf, strlen(buf));
  if (Pibsta^ AND ERR) <> 0 THEN GPIBerr('Error performing AUTOSCALE command');

                (* Send out confirming message *)
  msg := 'Executing AUTOSCALE';
  ListBox1.Items.Add(msg);
  ListBox1.Refresh;
  SleepEx (5000, FALSE);      (* give it 5 seconds *)

                (* refresh display data *)
  if channel_1_display or channel_1_fft or (mixed_display_mode > 0) then
     begin
     channel_number := 1;
     Get_Information_About_Waveform (channel_number); { read preamble, etc. }
     get_waveform_data ( channel_number );   { gets data, converts, computes FFT }
     end;

   if channel_2_display or channel_2_fft or (mixed_display_mode > 0) then
     begin
     channel_number := 2;
     Get_Information_About_Waveform (channel_number); { read preamble, etc. }
     get_waveform_data ( channel_number );   { gets data, converts, computes FFT }
     end;
  paintbox1.refresh;    { call display routine }
end;





(* =====================================================================
 *                    Procedure TForm1.RadioGroup1Click
 * This procedure is activated when the one of the radio buttons in Radio
 * Group 1 is clicked. The appropriate waveform is displayed
 * Refreshing must be done with the REFRESH button.
 * ===================================================================== *)


procedure TForm1.RadioGroup1Click ( Sender: TObject );

var channel_number: integer;

begin
 channel_number := 1;
 if not initialized then initialize_hp54200A;        { make sure initialization complete }
 channel_1_display := false;
 channel_1_fft := false;
 mixed_display_mode := 0;
{ radiogroup3.ItemIndex := 3;  }

 case RadioGroup1.ItemIndex of
      0: begin
         Get_Information_About_Waveform (channel_number); { read preamble, etc. }
         channel_1_display := true;
         get_waveform_data ( channel_number );   { gets data, converts, computes FFT }
         end;
      1: begin
         Get_Information_About_Waveform (channel_number); { read preamble, etc. }
         channel_1_FFT := true;
         get_waveform_data ( channel_number );
         end;
      2: begin
         Get_Information_About_Waveform (channel_number); { read preamble, etc. }
         channel_1_display := true;
         channel_1_FFT := true;
         get_waveform_data ( channel_number );
         end;
      end;
 paintbox1.refresh;    { call display routine }
end;  { TForm1.RadioGroup1 }


(* =====================================================================
 *                    Procedure TForm1.RadioGroup2Click
 * This procedure is activated when the one of the radio buttons in Radio
 * Group 2 is clicked. The appropriate waveform is displayed
 * Refreshing must be done with the REFRESH button.
 * ===================================================================== *)


procedure TForm1.RadioGroup2Click ( Sender: TObject );

var channel_number: integer;

begin
 channel_number := 2;
 if not initialized then initialize_hp54200A;        { make sure initialization complete }
 channel_2_display := false;
 channel_2_fft := false;
 mixed_display_mode := 0;
{ radiogroup3.ItemIndex := 3; }

 case RadioGroup2.ItemIndex of
      0: begin
         Get_Information_About_Waveform (channel_number); { read preamble, etc. }
         channel_2_display := true;
         get_waveform_data ( channel_number );   { gets data, converts, computes FFT }
         end;
      1: begin
         Get_Information_About_Waveform (channel_number); { read preamble, etc. }
         channel_2_FFT := true;
         get_waveform_data ( channel_number );
         end;
      2: begin
         Get_Information_About_Waveform (channel_number); { read preamble, etc. }
         channel_2_display := true;
         channel_2_FFT := true;
         get_waveform_data ( channel_number );
         end;
      end;
 paintbox1.refresh;    { call display routine }
end;

(* =====================================================================
 *                    Procedure TForm1.RadioGroup3Click
 * This procedure is activated when the one of the radio buttons in Radio
 * Group 3 is clicked. The appropriate waveform is displayed.  Any displays
 * from RadioGroup1 or 2 are disabled.  No FFT is done on the combined waveforms.
 * Refreshing must be done with the REFRESH button.
 * ===================================================================== *)


procedure TForm1.RadioGroup3Click ( Sender: TObject );

var channel_number: integer;
    i: integer;

begin
 if not initialized then initialize_hp54200A;        { make sure initialization complete }
          (* disable individual channel displays *)
 channel_1_fft := false;
 channel_2_fft := false;

         { Some glitch--this type of code, modifying button settings, trashes display  }

          (* turn off the other display modes *)
 with RadioGroup1 do
      begin
      For i := 0 to 2 do TRadioButton(components[i]).Checked := False;
      TRadioButton(components[3]).Checked := True;     (* show it turned off *)
      end;
 with RadioGroup2 do
      begin
      For i := 0 to 2 do TRadioButton(components[i]).Checked := False;
      TRadioButton(components[3]).Checked := True;     (* show it turned off *)
      end;



          (* get channel info for both channels -- refresh if both not in memory *)
 if (not channel_1_display or not channel_2_display) then for channel_number := 1 to 2 do
     begin
     Get_Information_About_Waveform (channel_number); { read preamble, etc. }
     get_waveform_data ( channel_number );   { gets data, converts, computes FFT }
     end;
 mixed_display_mode := 0;

 case RadioGroup3.ItemIndex of
      0: mixed_display_mode := 1;        { both channels on same graph, separate }
      1: mixed_display_mode := 2;        { channels added together }
      2: mixed_display_mode := 3;        { channel 2 subtracted from channel 1 }
      end;

 channel_1_display := false;
 channel_2_display := false;
 paintbox1.refresh;    { call display routine }
end;

(* =====================================================================
 *                    Procedure TForm1.RadioGroup4Click
 * This procedure is activated when the one of the radio buttons in Radio
 * Group 4 is clicked. If needed, waveform is inverted. Screen is not repainted.
 * ===================================================================== *)


procedure TForm1.RadioGroup4Click ( Sender: TObject );

var channel_number: integer;

begin
 channel_number := 1;
 with waveform_data [ channel_number ] do
      case RadioGroup4.ItemIndex of
           0: channel_inverted := false;
           1: channel_inverted := true;
      end;
  Get_Information_About_Waveform (channel_number); { read preamble, etc. }
  get_waveform_data ( channel_number );   { gets data, converts, computes FFT }
  paintbox1.refresh;    { call display routine }
end;

(* =====================================================================
 *                    Procedure TForm1.RadioGroup5Click
 * This procedure is activated when the one of the radio buttons in Radio
 * Group 5 is clicked. Coupling at the scope is set as requested, AC or DC.
 * ===================================================================== *)


procedure TForm1.RadioGroup5Click ( Sender: TObject );

var channel_number: integer;
    buf     : data_buffer;   (* R/W buffer.               *)

begin
  channel_number := 1;
  with waveform_data [ channel_number ] do
       begin
       case RadioGroup5.ItemIndex of
            0: coupling := 2;  { AC }
            1: coupling := 1;  { DC }
            end;
       if not initialized then initialize_hp54200A;        { make sure initialization complete }
       buf := 'CHANNEL 1';
       ibwrt(Dev, buf, strlen(buf));
       channel_subsystem_addressed := true;
       waveform_subsystem_addressed := false;
       measure_subsystem_addressed := false;
       SleepEx ( command_delay, FALSE );
       if coupling = 2
          then buf := 'COUPLING AC'
          else buf := 'COUPLING DC';
       end;
  ibwrt(Dev, buf, strlen(buf));
  SleepEx ( command_delay, FALSE );
  if (Pibsta^ AND ERR) <> 0 THEN GPIBerr('Error setting coupling for channel 1');

                (* Send out confirming message *)
  msg := 'Setting Channel 1 Coupling';
  ListBox1.Items.Add(msg);
  ListBox1.Refresh;
                (* redisplay *)
  Get_Information_About_Waveform (channel_number); { read preamble, etc. }
  get_waveform_data ( channel_number );   { gets data, converts, computes FFT }
  paintbox1.refresh;    { call display routine }
end;




(* =====================================================================
 *                    Procedure TForm1.RadioGroup6Click
 * This procedure is activated when the one of the radio buttons in Radio
 * Group 4 is clicked. If needed, waveform is inverted. Screen is not repainted.
 * ===================================================================== *)


procedure TForm1.RadioGroup6Click ( Sender: TObject );

var channel_number: integer;

begin
 channel_number := 2;
 with waveform_data [ channel_number ] do
      case RadioGroup6.ItemIndex of
           0: channel_inverted := false;
           1: channel_inverted := true;
      end;
 Get_Information_About_Waveform (channel_number); { read preamble, etc. }
 get_waveform_data ( channel_number );   { gets data, converts, computes FFT }
 paintbox1.refresh;    { call display routine }
end;





(* =====================================================================
 *                    Procedure TForm1.RadioGroup7Click
 * This procedure is activated when the one of the radio buttons in Radio
 * Group 7 is clicked. Coupling at the scope is set as requested, AC or DC.
 * ===================================================================== *)


procedure TForm1.RadioGroup7Click ( Sender: TObject );

var channel_number: integer;
    buf     : data_buffer;   (* R/W buffer.               *)

begin
  channel_number := 2;
  with waveform_data [ channel_number ] do
       begin
       case RadioGroup7.ItemIndex of
            0: coupling := 2;  { AC }
            1: coupling := 1;  { DC }
       end;
       if not initialized then initialize_hp54200A;        { make sure initialization complete }
       buf := 'CHANNEL 2';
       ibwrt(Dev, buf, strlen(buf));    (* initialize channel subsystem *)
       channel_subsystem_addressed := true;
       waveform_subsystem_addressed := false;
       measure_subsystem_addressed := false;
       SleepEx ( command_delay, FALSE );
       if coupling = 2
          then buf := 'COUPLING AC'
          else buf := 'COUPLING DC';
       end;
  ibwrt(Dev, buf, strlen(buf));
  if (Pibsta^ AND ERR) <> 0 THEN GPIBerr('Error setting coupling for channel 2');

                (* Send out confirming message *)
  msg := 'Setting Channel 2 Coupling';
  ListBox1.Items.Add(msg);
  ListBox1.Refresh;
                (* redisplay *)
  Get_Information_About_Waveform (channel_number); { read preamble, etc. }
  get_waveform_data ( channel_number );   { gets data, converts, computes FFT }
  paintbox1.refresh;    { call display routine }
end;



(* =====================================================================
 *                    Procedure TForm1.RadioGroup8Click
 * This procedure is activated when the one of the radio buttons in Radio
 * Group 4 is clicked. Selects averaging for Channel 1.
 * ===================================================================== *)


procedure TForm1.RadioGroup8Click ( Sender: TObject );

var channel_number: integer;

begin
  channel_number := 1;
  with waveform_data [ channel_number ] do
       begin
         case RadioGroup8.ItemIndex of
              0: averaging := false;
              1: averaging := true;
              end;

               (* Send out confirming message *)
         if averaging then
            begin
            msg := 'Averaging '+InttoStr(number_to_average) + ' data sets for ch 1';
            ListBox1.Items.Add(msg);
            ListBox1.Refresh;
            end;
         end;
  Get_Information_About_Waveform (channel_number); { read preamble, etc. }
  get_waveform_data ( channel_number );   { gets data, converts, computes FFT }
  paintbox1.refresh;    { call display routine }
end;




(* =====================================================================
 *                    Procedure TForm1.RadioGroup9Click
 * This procedure is activated when the one of the radio buttons in Radio
 * Group 4 is clicked. Selects averaging for Channel 2.
 * ===================================================================== *)


procedure TForm1.RadioGroup9Click ( Sender: TObject );

var channel_number: integer;

begin
  channel_number := 2;
  with waveform_data [ channel_number ] do
       begin
         case RadioGroup9.ItemIndex of
              0: averaging := false;
              1: averaging := true;
         end;
               (* Send out confirming message *)
         if averaging then
            begin
            msg := 'Averaging '+InttoStr(number_to_average) + ' data sets for ch 2';
            ListBox1.Items.Add(msg);
            ListBox1.Refresh;
            end;
         end; {with}
  Get_Information_About_Waveform (channel_number); { read preamble, etc. }
  get_waveform_data ( channel_number );   { gets data, converts, computes FFT }
  paintbox1.refresh;    { call display routine }
end;




(* =====================================================================
 *                    Procedure TForm1.Get_output_format
 * This procedure uses a drop-down list to get the desired output format.
 * Currently active graphs (or corresponding data) will be output when
 * user pushes OUTPUT button
 * ===================================================================== *)

procedure TForm1.Get_output_format ( Sender: TObject );

begin
  Metafile_output := false;
  Copy_to_clipboard:= false;
  Textfile_output := false;
  case ComboBox1.ItemIndex of
       0: Metafile_output := true;
       1: Copy_to_clipboard := true;
       2: Textfile_output := true;
       end;
  output_format_selected := true;
end;



(* =====================================================================
 *                    Procedure TForm1.Get_ch_1_averaging_times
 * This procedure uses a drop-down list to get the desired number of times
 * to average waveform.  Default is 5.
 * ===================================================================== *)

procedure TForm1.Get_ch_1_averaging_times ( Sender: TObject );

begin
  with waveform_data [1] do
  case ComboBox2.ItemIndex of
       0: number_to_average := 2;
       1: number_to_average := 5;
       2: number_to_average := 10;
       3: number_to_average := 20;
       4: number_to_average := 50;
       end;

end;


(* =====================================================================
 *                    Procedure TForm1.Get_ch_2_averaging_times
 * This procedure uses a drop-down list to get the desired number of times
 * to average waveform.  Default is 5.
 * ===================================================================== *)

procedure TForm1.Get_ch_2_averaging_times ( Sender: TObject );

begin
  with waveform_data [2] do
  case ComboBox3.ItemIndex of
       0: number_to_average := 2;
       1: number_to_average := 5;
       2: number_to_average := 10;
       3: number_to_average := 20;
       4: number_to_average := 50;
       end;

end;



(* =====================================================================
 *                    Procedure TForm1.Get_output_format
 * This procedure uses a drop-down list to get the desired output format.
 * Currently active graphs (or corresponding data) will be output when
 * user pushes OUTPUT button
 * ===================================================================== *)

Procedure TForm1.Output_currently_active_graphs ( Sender: TObject);

var ok: boolean;

begin
  if output_format_selected then
     begin
     if metafile_output then xysaveasmetafile ( '', msgon, ok );
     if copy_to_clipboard then
        begin
        xycopytoclipboard;
        showmessage ( 'Graph copied to clipboard' );
        end;
     if textfile_output then output_data_file;
     end
  else MessageDlg('No output format currently selected',mtInformation,[mbOK],0);
end;  { Output_currently_active_graphs }

end.
